struct sj_argv_memory *create_argv_memory(pid_t pid,
					  char argv[MAX_KEY_ELEMENTS +
						    1][BUFFSIZE]);
int             destroy_sj_argv_memory(pid_t pid);
struct sj_argv_memory *get_argv_memory(pid_t pid);


struct sj_argv_memory *
create_argv_memory(pid_t pid, char argv[MAX_KEY_ELEMENTS + 1][BUFFSIZE])
{
    struct sj_argv_memory *memory;
    int             i,
                    j;

    memory = kmalloc(sizeof(*memory), GFP_KERNEL);

    if (memory) {
	memory->pid = pid;
	for (i = 0; i < MAX_KEY_ELEMENTS + 1; i++)
	    for (j = 0; j < BUFFSIZE; j++)
		memory->argv[i][j] = argv[i][j];
	memory->next = sj_argv_memory_hash[memory->pid % SJ_MEMORY_HASH];
	sj_argv_memory_hash[memory->pid % SJ_MEMORY_HASH] = memory;
#ifdef PROFILE
	printk("(STJUDE) CREATE_ARGV_MEMORY: Memory of %d created.\n",
	       pid);
#endif
	return memory;
    } else {
	printk("<1>(STJUDE) Unable to allocate memory structure\n");
    }
    return NULL;

}

/*
 * 
 * Get a priv record based on its pid.  (DUH!)
 * 
 */


struct sj_argv_memory *
get_argv_memory(pid_t pid)
{
    struct sj_argv_memory *memory;

#ifdef PROFILE
    printk("(STJUDE) GET_ARGV_MEORY: getting memory of %d.\n", pid);
#endif

    memory = sj_argv_memory_hash[pid % SJ_MEMORY_HASH];

    while (memory != NULL) {

	if (memory->pid == pid)
	    break;

	memory = memory->next;

    }

    if (memory == NULL) {
	struct sj_argv_memory *parent_memory;

	parent_memory = get_argv_memory(current->p_pptr->pid);
	if (parent_memory) {
	    memory = create_argv_memory(current->pid, parent_memory->argv);
	    if (!memory)
		printk
		    ("(StJude) sj_fork: Something bad happened, I can't remember argv's.\n");
	} else {
	    printk
		("(StJude) suid_hack: Something bad happened. I can't find my parent's (%d) memory.\n",
		 current->p_pptr->pid);
	}
    }
#ifdef PROFILE
    if (memory->pid == pid)
	printk("(STJUDE) GET_ARGV_MEORY: Memory of %d gotten.\n", pid);
    else
	printk("(STJUDE) GET_ARGV_MEORY: Memory of %d NOT gotten.\n", pid);
#endif
    return memory;
}

int
destroy_argv_memory(pid_t pid)
{
    struct sj_argv_memory *memory;
    struct sj_argv_memory *prev;

#ifdef PROFILE
    printk("(STJUDE) DESTROY_ARGV_MEMORY: Set to Destroy PID %d.\n", pid);
#endif

    prev = NULL;
    memory = sj_argv_memory_hash[pid % SJ_MEMORY_HASH];

    while (memory != NULL) {

	if (memory->pid == pid)
	    break;

	prev = memory;
	memory = memory->next;

    }


    if (memory != NULL) {
	if (prev != NULL)
	    prev->next = memory->next;
	else
	    sj_argv_memory_hash[pid % SJ_PRIV_HASH] = memory->next;

	kfree(memory);

    } else {
	return 0;
    }				/*
				 * * Tried to delete a record that doesn't
				 * exit 
				 */

    return 1;
}
